; Sprite Demo / Test
; Charles Wieland, 1/29/1998
; junkosoft.org

	processor 6502
	include vcs.h

	org $F000

NUMSCAN  = $80 ; Number of scanlines per row
NUMROW   = $81 ; Number of row per screen
CURCOLOR = $82 ; Curcolor of playfield / background
SPRPOS   = $83 ; Current position of sprite
SPRSIZE  = $84 ; Size of sprite in scanlines
SPRCNT   = $85 ; Sprite line count
LINECNT  = $86 ; Counter used to draw sprite
CURSCAN  = $87 ; Current Scanline
CURROW   = $88 ; Current Row

Start	SEI
	CLD
	LDX #$FF
	TXS
	LDA #$00

clrstk	STA $00,X
	DEX
	BNE clrstk
	STX CURCOLOR	; Set current color to black
	LDA #$08	; 08 scanlines per row
	STA NUMSCAN
	LDA #$07
	STA SPRSIZE	; Size of sprite (8 scanlines 0-7)
	LDA #$17	; 24 row per screen (0-23)
	STA NUMROW
	LDA #$01
	STA CTRLPF
	LDA #$45	; Initial Sprite location
	STA SPRPOS
	LDA #$0F
	STA COLUP0	; White sprite color

main	JSR vblank
	JSR vars
	JSR draw
	JSR oscan
	JMP main

vblank	LDA #$02
	STA WSYNC
	STA WSYNC
	STA WSYNC
	STA VSYNC
	STA WSYNC
	STA WSYNC
	STA HMOVE
	LDA #$2C
	STA TIM64T
	LDA #$00
	STA CXCLR
	STA WSYNC
	STA VSYNC
	LDA #$C0 ; Reset Line Counter to 192
	STA LINECNT
	RTS

vars	LDA #$00
	STA COLUBK
	LDA SWCHB
	AND #$08 ; Check for B/W - Color Switch
	BEQ bw

color	INC CURCOLOR
	LDX CURCOLOR
	LDA SWCHB
	AND #$40
	BNE back
	STX COLUPF
	RTS

bw	INC CURCOLOR
	LDX CURCOLOR	
	TXA
	CLV
	CMP #$0E
	BPL reset
	LDA SWCHB
	AND #$40
	BNE back
	STX COLUPF
	RTS

reset	LDX #$00
	STX CURCOLOR
	LDA SWCHB
	AND #$40
	BNE back
	STX COLUPF
	RTS

back	STX COLUBK
	LDX #$00
	STX COLUPF
	RTS

draw	LDA INTIM
	BNE draw
	STA WSYNC
	STA VBLANK
	LDA SPRSIZE
	STA SPRCNT 
	LDY NUMSCAN
	STY CURSCAN
	LDX NUMROW
	STX CURROW

loadpf  LDA pf0data,X
	STA PF0
	LDA pf1data,X
	STA PF1
	LDA pf2data,X
	STA PF2

chkspr	DEC LINECNT
	LDA LINECNT
	CMP SPRPOS
	BMI drawspr

doline	STA WSYNC
	DEC CURSCAN
	BNE chkspr

newrow	LDA CURROW
	BEQ clear
	DEC CURROW
	LDY NUMSCAN
	STY CURSCAN
	LDX CURROW
	JMP loadpf

drawspr	LDX SPRCNT
	BEQ donespr
	LDA sprdata,X
	STA GRP0
	DEC SPRCNT
	JMP doline

donespr	LDA #$00
	STA GRP0
	JMP doline

clear   LDA #$02
	STA WSYNC
	STA VBLANK
	RTS

oscan	LDA SWCHA
	CMP #$EF
	BEQ up
	CMP #$DF
	BEQ down
	CMP #$BF
	BEQ left
	CMP #$7F
	BEQ right
	LDA #$00
	STA HMP0
	JMP over

up	INC SPRPOS
	JMP over

down	DEC SPRPOS
	JMP over

left	LDA #$10
	STA HMP0
	JMP over

right	LDA #$F0
	STA HMP0

over	LDX #$1E
doscan  STA WSYNC
	DEX
        BNE doscan
        RTS

	org $FE70
	; Sprite Data
sprdata	.byte $AA,$55,$AA,$55,$AA,$55,$AA,$55

	org $FF00
	; Playfield Data
pf0data .byte $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
	.byte $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
	.byte $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
pf1data .byte $00,$00,$00,$06,$03,$01,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
pf2data .byte $00,$00,$00,$C0,$C0,$C1,$C3,$C6
	.byte $C6,$CC,$CC,$D8,$D8,$D8,$D8,$D8
	.byte $D8,$D8,$D8,$D8,$00,$00,$00,$00

	org $FFFC
	.word Start
	.word Start	