; Sprite Demo / Test with somewhat fixed sprite limits
; Charles Wieland, 3/1/1998
; junkosoft.org

	processor 6502
	include vcs.h

	org $F000

SPRPOS   = $80 ; Current position of sprite
SPRSIZE  = $81 ; Size of sprite in scanlines
SPRCNT   = $82 ; Sprite line count
LINECNT  = $83 ; Counter used to draw sprite

; Main Program and Initialization

Start	SEI
	CLD
	LDX #$FF
	TXS
	LDA #$00

clrstk	STA $00,X
	DEX
	BNE clrstk
	LDA #$07
	STA SPRSIZE	; Size of sprite (8 scanlines 0-7)
	LDA #$5C	; Initial Sprite location
	STA SPRPOS
	LDA #$00
	STA COLUPF	; Black playfield color	

main	JSR vblank
	JSR vars
	JSR draw
	JSR oscan
	JMP main

; Vertical Blank Handler

vblank	LDA #$02
	STA WSYNC
	STA WSYNC
	STA WSYNC
	STA VSYNC
	STA WSYNC
	STA WSYNC
	STA HMOVE
	LDA #$2C
	STA TIM64T
	LDA #$00
	STA CXCLR
	STA WSYNC
	STA VSYNC
	LDA #$C0 ; Reset Line Counter to 192
	STA LINECNT
	RTS

; Color Setup Routine

vars	LDA SWCHB
	AND #$08 ; Check for B/W - Color Switch
	BEQ bw

color	LDX #$32
	STX COLUP0
	RTS

bw	LDX #$08
	STX COLUP0
	RTS

; Screen Draw routine (this really sucks)

draw	LDA INTIM
	BNE draw
	STA WSYNC
	STA VBLANK
	LDA SPRSIZE
	STA SPRCNT 

chkspr	LDA #$00
	STA GRP0
	DEC LINECNT
	LDA LINECNT
	BEQ clear
	CMP SPRPOS	; This causes many extra lines of code
	BMI drawspr	; to be run after the sprite is drawn
	LDA SPRCNT	; These two new lines might help the speed problem
	BMI doline	; after the sprite, will slow down before drawn
 
doline	STA WSYNC
	JMP chkspr

drawspr	LDX SPRCNT
	BMI doline
	LDA sprdata,X
	STA GRP0
	DEC SPRCNT
	JMP doline

clear   LDA #$02
	STA WSYNC
	STA VBLANK
	RTS

; Overscan and Joystick routine

oscan	LDA SWCHA
	CMP #$BF
	BEQ left
	CMP #$7F
	BEQ right

	LDA #$00
	STA HMP0

	LDA SWCHA
	CMP #$EF
	BEQ up
	CMP #$DF
	BEQ down
	JMP over

up	LDA SPRPOS
	CMP #$C0
	BEQ over
	INC SPRPOS
	JMP over

down	LDA SPRPOS
	CMP #$01
	BEQ over
	DEC SPRPOS
	JMP over

left	LDA #$10
	STA HMP0
	JMP over

right	LDA #$F0
 	STA HMP0

over	LDX #$1E
doscan  STA WSYNC
	DEX
        BNE doscan
        RTS

	org $FE70
	; Sprite Data
sprdata	.byte $FF,$FF,$7E,$7E,$3C,$3C,$18,$18
;	.byte $18,$18,$3C,$E7,$E7,$3C,$18,$18

	org $FF00
	; Playfield Data
pf0data .byte $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
	.byte $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
	.byte $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
pf1data .byte $00,$00,$00,$06,$03,$01,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
pf2data .byte $00,$00,$00,$C0,$C0,$C1,$C3,$C6
	.byte $C6,$CC,$CC,$D8,$D8,$D8,$D8,$D8
	.byte $D8,$D8,$D8,$D8,$00,$00,$00,$00

	org $FFFC
	.word Start
	.word Start	