; Sprite Demo / Test with rewritten draw function
; Charles Wieland, 3/4/1998
; junkosoft.org

	processor 6502
	include vcs.h

	org $F000

TOPPOS	= $80 ; Number of lines at top
BOTPOS	= $81 ; Number of lines at bottom
SPRSIZE	= $82 ; Size of sprite in scanlines

; Main Program and Initialization

Start   SEI
	CLD
	LDX #$FF
	TXS
	LDA #$00

clrstk  STA $00,X
	DEX
	BNE clrstk
	LDA #$07
	STA SPRSIZE     ; Size of sprite (8 scanlines)
	LDA #$5C        ; Initial Sprite location
	STA TOPPOS
	STA BOTPOS
	LDA #$00
	STA COLUPF      ; Black playfield color

	LDX #$5C
topfrm  STA WSYNC
	DEX
	BNE topfrm
	LDX #$90
delay   DEX
	BNE delay
	NOP
	STA RESP0
	STA WSYNC
	LDX #$63
btmfrm  STA WSYNC
	DEX
	BNE btmfrm

main    JSR vblank
	JSR vars
	JSR draw
	JSR oscan
	JMP main

; Vertical Blank Handler

vblank  LDA #$02
	STA WSYNC
	STA WSYNC
	STA WSYNC
	STA VSYNC
	STA WSYNC
	STA WSYNC
	STA HMOVE
	LDA #$2C
	STA TIM64T
	LDA #$00
	STA CXCLR       
	STA WSYNC
	STA VSYNC
	RTS

; Color Setup Routine

vars    LDA SWCHB
	AND #$08 ; Check for B/W - Color Switch
	BEQ bw

color   LDX #$32
	STX COLUP0
	RTS

bw      LDX #$08
	STX COLUP0
	RTS

; Screen Draw routine

draw    LDA INTIM
	BNE draw
	STA WSYNC
	STA VBLANK
	LDX TOPPOS
drawtop STA WSYNC
	DEX
	BNE drawtop
	LDX SPRSIZE     
drawspr LDA sprdata,X
	STA GRP0
	STA WSYNC
	DEX
	BNE drawspr
	LDA sprdata
	STA GRP0
	STA WSYNC
	STX GRP0
	LDX BOTPOS
drawbot STA WSYNC
	DEX
	BNE drawbot
	RTS

; Overscan and Joystick routine

oscan   LDA SWCHA
	CMP #$BF
	BEQ left
	CMP #$7F
	BEQ right

	LDA #$00
	STA HMP0

	LDA SWCHA
	CMP #$EF
	BEQ up
	CMP #$DF
	BEQ down
	JMP over

up      LDA TOPPOS
	CMP #$01
	BEQ over
	DEC TOPPOS
	INC BOTPOS
	JMP over

down    LDA BOTPOS
	CMP #$01
	BEQ over
	DEC BOTPOS
	INC TOPPOS
	JMP over

left    LDA #$10
	STA HMP0
	JMP over

right   LDA #$F0
	STA HMP0

over    LDX #$1E
doscan  STA WSYNC
	DEX
	BNE doscan
	RTS

	org $FE70
	; Sprite Data
sprdata .byte $FF,$FF,$7E,$7E,$3C,$3C,$18,$18 ; Triangle
	.byte $18,$18,$3C,$E7,$E7,$3C,$18,$18 ; Crosshair
	.byte $03,$07,$0E,$9C,$F8,$F0,$F0,$F8 ; Mouse Pointer

	org $FF00
	; Playfield Data
pf0data .byte $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
	.byte $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
	.byte $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
pf1data .byte $00,$00,$00,$06,$03,$01,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
pf2data .byte $00,$00,$00,$C0,$C0,$C1,$C3,$C6
	.byte $C6,$CC,$CC,$D8,$D8,$D8,$D8,$D8
	.byte $D8,$D8,$D8,$D8,$00,$00,$00,$00

	org $FFFC
	.word Start
	.word Start     
