; Sprite Demo / Test with sprite select
; Charles Wieland, 3/4/1998
; junkosoft.org

	processor 6502
	include vcs.h

	org $F000

TOPPOS  = $80 ; Number of lines at top
BOTPOS  = $81 ; Number of lines at bottom
SPRSIZE = $82 ; Size of sprite in scanlines
CURSPR  = $83 ; Current Sprite
FRMCNT	= $84 ; Frame Count for animation

; Main Program and Initialization

Start   SEI
	CLD
	LDX #$FF
	TXS
	LDA #$00

clrstk  STA $00,X
	DEX
	BNE clrstk
	LDA #$07
	STA SPRSIZE     ; Size of sprite (8 scanlines)
	LDA #$5C        ; Initial Sprite location
	STA TOPPOS
	STA BOTPOS
	LDA #$00
	STA COLUPF      ; Black playfield color
	STA CURSPR	; Set current sprite to 0
	LDA #$14
	STA FRMCNT	; Set Frame number to 20

	LDX #$5C
topfrm  STA WSYNC
	DEX
	BNE topfrm
	LDX #$90
delay   DEX
	BNE delay
	NOP
	STA RESP0
	STA WSYNC
	LDX #$63
btmfrm  STA WSYNC
	DEX
	BNE btmfrm

main    JSR vblank
	JSR vars
	JSR draw
	JSR oscan
	JMP main

; Vertical Blank Handler

vblank  LDA #$02
	STA WSYNC
	STA WSYNC
	STA WSYNC
	STA VSYNC
	STA WSYNC
	STA WSYNC
	STA HMOVE
	LDA #$2C
	STA TIM64T
	LDA #$00
	STA CXCLR       
	STA WSYNC
	STA VSYNC
	RTS

; Color Setup Routine

vars    LDA SWCHB
	AND #$08 ; Check for B/W - Color Switch
	BEQ bw

color   LDX #$32
	STX COLUP0
	JMP select

bw      LDX #$08
	STX COLUP0
	
select  LDA INPT4
	BPL chkani
	RTS

chkani	LDA FRMCNT
	BEQ chgfrm
	DEC FRMCNT
	RTS
	
chgfrm	LDA #$14
	STA FRMCNT
	LDA CURSPR
	CMP #$20
	BEQ rstspr
	ADC #$08
	STA CURSPR
	RTS

rstspr  LDA #$00
	STA CURSPR
	RTS
	
; Screen Draw routine

draw    LDA INTIM
	BNE draw
	STA WSYNC
	STA VBLANK
	LDX TOPPOS
drawtop STA WSYNC
	DEX
	BNE drawtop
	LDA SPRSIZE
	TAX
	ADC CURSPR
	TAY
	DEY
drawspr LDA sprdata,Y
	STA GRP0
	STA WSYNC
	DEY
	DEX
	BNE drawspr
	LDA sprdata,Y
	STA GRP0
	STA WSYNC
	STX GRP0
	LDX BOTPOS
drawbot STA WSYNC
	DEX
	BNE drawbot
	RTS

; Overscan and Joystick routine

oscan   LDA SWCHA
	CMP #$BF
	BEQ left
	CMP #$7F
	BEQ right

	LDA #$00
	STA HMP0

	LDA SWCHA
	CMP #$EF
	BEQ up
	CMP #$DF
	BEQ down
	JMP over

up      LDA TOPPOS
	CMP #$01
	BEQ over
	DEC TOPPOS
	INC BOTPOS
	JMP over

down    LDA BOTPOS
	CMP #$01
	BEQ over
	DEC BOTPOS
	INC TOPPOS
	JMP over

left    LDA #$10
	STA HMP0
	JMP over

right   LDA #$F0
	STA HMP0

over    LDX #$1E
doscan  STA WSYNC
	DEX
	BNE doscan
	RTS

	org $FE70
	; Sprite Data
sprdata .byte $FF,$FF,$7E,$7E,$3C,$3C,$18,$18 ; Triangle
	.byte $18,$18,$3C,$E7,$E7,$3C,$18,$18 ; Crosshair
	.byte $03,$07,$0E,$9C,$F8,$F0,$F0,$F8 ; Mouse Pointer
	.byte $3C,$42,$5A,$FF,$FF,$5A,$7E,$3C ; Happy Face
	.byte $00,$E7,$A1,$A1,$27,$24,$24,$F7 ; Junkosoft Logo

	org $FF00
	; Playfield Data
pf0data .byte $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
	.byte $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
	.byte $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
pf1data .byte $00,$00,$00,$06,$03,$01,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
pf2data .byte $00,$00,$00,$C0,$C0,$C1,$C3,$C6
	.byte $C6,$CC,$CC,$D8,$D8,$D8,$D8,$D8
	.byte $D8,$D8,$D8,$D8,$00,$00,$00,$00

	org $FFFC
	.word Start
	.word Start     
