; Sprite Demo / Test with 2 players, playfield and some collision
; Charles Wieland, 3/15/1998
; junkosoft.org

	processor 6502
	include vcs.h

	org $F000

POS0     = $80 ; Top Position of Player 0
POS1     = $81 ; Top Position of Player 1
SPRSIZE0 = $82 ; Size of sprite in scanlines p0
SPRSIZE1 = $83 ; Size of sprite in scanlines p1
CURSPR0  = $84 ; Current sprite for p0
CURSPR1  = $85 ; Current sprite for p1
FRMCNT0  = $86 ; Frame count for p0
FRMCNT1  = $87 ; Frame count for p1
LINEP0   = $88 ; Line count for drawing p0
LINEP1   = $89 ; Line count for drawing p1
PFSCAN   = $8A ; Number of scanlines per block of pf
PFROW    = $8B ; Number of rows of pf
VERTFRM0 = $8C ; Vertical Frame for p0
VERTFRM1 = $8D ; Vertical Frame for p1

; Main Program and Initialization

Start   SEI
	CLD
	LDX #$FF
	TXS
	LDA #$00

clrstk  STA $00,X
	DEX
	BNE clrstk
	LDX #$0A
	STX VERTFRM0
	STX VERTFRM1
	LDA #$08
	STA SPRSIZE0    ; Size of sprite p0 (8 scanlines)
	STA SPRSIZE1    ; Size of sprite p1 (8 scanlines)
	LDA #$10        ; Initial Sprite locations
	STA POS0
	STA POS1
	LDA #$01
	STA CTRLPF
	STA CURSPR0     ; Set current p0 sprite to 0
	STA CURSPR1     ; Set current p1 sprite to 0
	LDA #$14
	STA FRMCNT0     ; Set Frame number to 20
	STA FRMCNT1
	LDA #$0C
	STA AUDC0
	STA AUDF0
	
	LDX #$5C
topfrm  STA WSYNC
	DEX
	BNE topfrm
	LDX #$90
delay   DEX
	BNE delay
	STA RESP0
	NOP
	STA RESP1
	STA WSYNC
	LDX #$63
btmfrm  STA WSYNC
	DEX
	BNE btmfrm

main    JSR vblank
	JSR vars
	JSR draw
	JSR oscan
	JMP main

; Vertical Blank Handler

vblank  LDA #$02
	STA WSYNC
	STA WSYNC
	STA WSYNC
	STA VSYNC
	STA WSYNC
	STA WSYNC
	STA HMOVE
	LDA #$2C
	STA TIM64T
	LDA #$00
	STA WSYNC
	STA VSYNC
	; Check Collisions
	LDA CXPPMM
	AND #$80
	BNE p0p1
	LDA #$00
	STA CXCLR
	STA AUDV0
	RTS

p0p1    LDA #$FF
	STA AUDV0
	STA CXCLR
	RTS

; Color Setup Routine

vars    LDA SWCHB
	AND #$08 ; Check for B/W - Color Switch
	BEQ bw

color   LDX #$DD
	STX COLUP0
	LDX #$96
	STX COLUP1
	LDX #$42
	STX COLUPF
	LDX #$B2
	STX COLUBK
	JMP select

bw      LDX #$0A
	STX COLUP0
	LDX #$08
	STX COLUP1
	LDX #$04
	STX COLUPF
	LDX #$0F
	STX COLUBK
	
select  LDA INPT4
	BPL chkani
	RTS

chkani  LDA FRMCNT0
	BEQ chgfrm
	DEC FRMCNT0
	RTS
	
chgfrm  LDA #$14
	STA FRMCNT0
	LDA CURSPR0
	CMP #$20
	BEQ rstspr
	ADC #$08
	STA CURSPR0
	RTS

rstspr  LDA #$00
	STA CURSPR0
	RTS
	
; Screen Draw routine

draw    LDA INTIM
	BNE draw
	STA WSYNC
	STA VBLANK
	LDX #$18
loadpf  DEX
	LDY #$08
	LDA pf0data,X
	STA PF0
	LDA pf1data,X
	STA PF1
	LDA pf2data,X
	STA PF2
	LDA #$00
	STA GRP0
	STA GRP1
drawscr STA WSYNC
	DEY
	CPX POS0
	BEQ loadp0
return0 CPX POS1
	BEQ loadp1
return1 TYA
	BNE drawscr
	TXA
	BNE loadpf
clear   LDA #$02
	STA WSYNC
	STA VBLANK
	RTS

loadp0  LDA sprdata,Y
	STA GRP0
	JMP return0
	
loadp1  LDA sprdata,Y
	STA GRP1
	JMP return1

; Overscan and Joystick routine

oscan   LDA SWCHA
	AND #$F0
	CMP #$B0
	BEQ left0
	CMP #$70
	BEQ right0

	LDX #$00
	STX HMP0

	CMP #$E0
	BEQ up0
	CMP #$D0
	BEQ down0
	JMP joy1

up0     INC VERTFRM0
	LDA VERTFRM0
	CMP #$0F
	BEQ incp0
	JMP joy1

incp0   LDX #$00
	STX VERTFRM0
	LDA POS0
	CMP #$17
	BEQ clrup0
	INC POS0
	JMP joy1

clrup0  LDA #$00
	STA POS0
	JMP joy1

down0   DEC VERTFRM0
	LDA VERTFRM0
	BEQ decp0
	JMP joy1

decp0   LDX #$0F
	STX VERTFRM0
	LDA POS0
	BEQ clrdwn0
	DEC POS0
	JMP joy1

clrdwn0 LDA #$17
	STA POS0
	JMP joy1

left0   LDA #$10
	STA HMP0
	JMP joy1

right0  LDA #$F0
	STA HMP0

joy1    LDA SWCHA
	AND #$0F
	CMP #$0B
	BEQ left1
	CMP #$07
	BEQ right1

	LDX #$00
	STX HMP1

	CMP #$0E
	BEQ up1
	CMP #$0D
	BEQ down1
	JMP over

up1     INC VERTFRM1
	LDA VERTFRM1
	CMP #$0F
	BEQ incp1
	JMP over

incp1   LDX #$00
	STX VERTFRM1
	LDA POS1
	CMP #$17
	BEQ clrup1
	INC POS1
	JMP over

clrup1  LDA #$00
	STA POS1
	JMP over

down1   DEC VERTFRM1
	LDA VERTFRM1
	BEQ decp1
	JMP over

decp1   LDX #$0F
	STX VERTFRM1
	LDA POS1
	BEQ clrdwn1
	DEC POS1
	JMP over

clrdwn1 LDA #$17
	STA POS1
	JMP over

left1   LDA #$10
	STA HMP1
	JMP over

right1  LDA #$F0
	STA HMP1

over    LDX #$1E
doscan  STA WSYNC
	DEX
	BNE doscan
	RTS

	org $FE70
	; Sprite Data
sprdata .byte $FF,$FF,$7E,$7E,$3C,$3C,$18,$18 ; Triangle
	.byte $18,$18,$3C,$E7,$E7,$3C,$18,$18 ; Crosshair
	.byte $03,$07,$0E,$9C,$F8,$F0,$F0,$F8 ; Mouse Pointer
	.byte $3C,$42,$5A,$FF,$FF,$5A,$7E,$3C ; Happy Face
	.byte $00,$E7,$A1,$A1,$27,$24,$24,$F7 ; Junkosoft Logo

	org $FF00
	; Playfield Data
pf0data .byte $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
	.byte $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
	.byte $0F,$0F,$0F,$0F,$0F,$0F,$0F,$0F
pf1data .byte $00,$00,$00,$06,$03,$01,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
pf2data .byte $00,$00,$00,$C0,$C0,$C1,$C3,$C6
	.byte $C6,$CC,$CC,$D8,$D8,$D8,$D8,$D8
	.byte $D8,$D8,$D8,$D8,$00,$00,$00,$00

	org $FFFC
	.word Start
	.word Start     
